@@ -15,6 +15,7 @@ import threading
 import typing as tp
 import warnings
 from collections import ChainMap
+from pathlib import Path
 
 import xonsh.prompt.base as prompt
 from xonsh import __version__ as XONSH_VERSION
@@ -28,7 +29,7 @@ from xonsh.built_ins import XSH
 from xonsh.codecache import run_script_with_cache
 from xonsh.dirstack import _get_cwd
 from xonsh.events import events
-from xonsh.lazyasd import LazyBool, lazyobject
+from xonsh.lib.lazyasd import LazyBool, lazyobject
 from xonsh.platform import (
     BASH_COMPLETIONS_DEFAULT,
     DEFAULT_ENCODING,
@@ -41,6 +42,7 @@ from xonsh.platform import (
 from xonsh.tools import (
     DefaultNotGiven,
     DefaultNotGivenType,
+    EnvPath,
     adjust_shlvl,
     always_false,
     always_true,
@@ -53,10 +55,10 @@ from xonsh.tools import (
     dynamic_cwd_tuple_to_str,
     ensure_string,
     env_path_to_str,
-    executables_in,
     history_tuple_to_str,
     intensify_colors_on_win_setter,
     is_bool,
+    is_bool_or_int,
     is_bool_or_none,
     is_completion_mode,
     is_completions_display_value,
@@ -80,6 +82,7 @@ from xonsh.tools import (
     pathsep_to_upper_seq,
     print_color,
     print_exception,
+    print_warning,
     ptk2_color_depth_setter,
     seq_to_upper_pathsep,
     set_to_csv,
@@ -96,6 +99,8 @@ from xonsh.tools import (
     to_int_or_none,
     to_itself,
     to_logfile_opt,
+    to_ptk_cursor_shape,
+    to_ptk_cursor_shape_display_value,
     to_repr_pretty_,
     to_shlvl,
     to_tok_color_dict,
@@ -487,11 +492,11 @@ class LsColors(cabc.MutableMapping):
             )
         except (subprocess.CalledProcessError, FileNotFoundError, NotADirectoryError):
             return cls(cls.default_settings)
-        except OSError:
+        except OSError as e:
             # necessary to catch OSError: [WinError 740] The requested operation requires elevation
-            if ON_WINDOWS:
-                return cls(cls.default_settings)
-            raise
+            if not ON_WINDOWS:
+                print_warning(f"dircolors: {e}")
+            return cls(cls.default_settings)
         if not out:
             return cls(cls.default_settings)
         s = out.splitlines()[0]
@@ -565,7 +570,9 @@ DEFAULT_TITLE = "{current_job:{} | }{user}@{hostname}: {cwd} | xonsh"
 @default_value
 def xonsh_data_dir(env):
     """Ensures and returns the $XONSH_DATA_DIR"""
-    xdd = os.path.expanduser(os.path.join(env.get("XDG_DATA_HOME"), "xonsh"))
+    xdd = os.path.expanduser(
+        os.getenv("XONSH_DATA_DIR") or os.path.join(env.get("XDG_DATA_HOME"), "xonsh")
+    )
     os.makedirs(xdd, exist_ok=True)
     return xdd
 
@@ -573,7 +580,9 @@ def xonsh_data_dir(env):
 @default_value
 def xonsh_cache_dir(env):
     """Ensures and returns the $XONSH_CACHE_DIR"""
-    xdd = os.path.expanduser(os.path.join(env.get("XDG_CACHE_HOME"), "xonsh"))
+    xdd = os.path.expanduser(
+        os.getenv("XONSH_CACHE_DIR") or os.path.join(env.get("XDG_CACHE_HOME"), "xonsh")
+    )
     os.makedirs(xdd, exist_ok=True)
     return xdd
 
@@ -581,7 +590,10 @@ def xonsh_cache_dir(env):
 @default_value
 def xonsh_config_dir(env):
     """``$XDG_CONFIG_HOME/xonsh``"""
-    xcd = os.path.expanduser(os.path.join(env.get("XDG_CONFIG_HOME"), "xonsh"))
+    xcd = os.path.expanduser(
+        os.getenv("XONSH_CONFIG_DIR")
+        or os.path.join(env.get("XDG_CONFIG_HOME"), "xonsh")
+    )
     os.makedirs(xcd, exist_ok=True)
     return xcd
 
@@ -622,6 +634,11 @@ def xonshconfig(env):
     return xc
 
 
+def get_home_xonshrc_path():
+    """Cross-platform implementation of getting ``~/.xonshrc`` path."""
+    return str((Path("~") / ".xonshrc").expanduser())
+
+
 @default_value
 def default_xonshrc(env) -> "tuple[str, ...]":
     """
@@ -631,7 +648,7 @@ def default_xonshrc(env) -> "tuple[str, ...]":
     dxrc = (
         os.path.join(xonsh_sys_config_dir(env), "xonshrc"),
         os.path.join(xonsh_config_dir(env), "rc.xsh"),
-        os.path.expanduser("~/.xonshrc"),
+        get_home_xonshrc_path(),
     )
     # Check if old config file exists and issue warning
     old_config_filename = xonshconfig(env)
@@ -705,7 +722,7 @@ def default_prompt_fields(env):
     return prompt.PromptFields(XSH)
 
 
-VarKeyType = tp.Union[str, tp.Pattern]
+VarKeyType = tp.Union[str, tp.Pattern]  # noqa: UP007
 
 
 class Var(tp.NamedTuple):
@@ -740,22 +757,22 @@ class Var(tp.NamedTuple):
         a regex pattern to match for the given variable
     """
 
-    validate: tp.Optional[tp.Callable] = always_true
-    convert: tp.Optional[tp.Callable] = None
-    detype: tp.Optional[tp.Callable] = ensure_string
+    validate: tp.Callable | None = always_true
+    convert: tp.Callable | None = None
+    detype: tp.Callable | None = ensure_string
     default: tp.Any = DefaultNotGiven
     doc: str = ""
-    is_configurable: tp.Union[bool, LazyBool] = True
-    doc_default: tp.Union[str, DefaultNotGivenType] = DefaultNotGiven
+    is_configurable: bool | LazyBool = True
+    doc_default: str | DefaultNotGivenType = DefaultNotGiven
     can_store_as_str: bool = False
-    pattern: tp.Optional[VarKeyType] = None
+    pattern: VarKeyType | None = None
 
     @classmethod
     def with_default(
         cls,
         default: object,
         doc: str = "",
-        doc_default: tp.Union[str, DefaultNotGivenType] = DefaultNotGiven,
+        doc_default: str | DefaultNotGivenType = DefaultNotGiven,
         type_str: str = "",
         **kwargs,
     ):
@@ -806,9 +823,7 @@ class Xettings:
                 yield var.get_key(var_name), var
 
     @staticmethod
-    def _get_groups(
-        cls, _seen: tp.Optional[set["Xettings"]] = None, *bases: "Xettings"
-    ):
+    def _get_groups(cls, _seen: set["Xettings"] | None = None, *bases: "Xettings"):
         if _seen is None:
             _seen = set()
         subs = cls.__subclasses__()
@@ -851,28 +866,6 @@ class Xettings:
 class GeneralSetting(Xettings):
     """General"""
 
-    AUTO_CONTINUE = Var.with_default(
-        False,
-        "If ``True``, automatically resume stopped jobs when they are disowned. "
-        "When stopped jobs are disowned and this option is ``False``, a warning "
-        "will print information about how to continue the stopped process.",
-    )
-
-    COMMANDS_CACHE_SAVE_INTERMEDIATE = Var.with_default(
-        False,
-        "If enabled, the CommandsCache is saved between runs and can reduce the startup time.",
-    )
-
-    ENABLE_COMMANDS_CACHE = Var(
-        default=True,
-        doc="command names in a directory are cached when enabled. "
-        "On some platforms it may not be accurate enough"
-        "(e.g. Windows, Linux save mtime in seconds). "
-        "Setting it to False would disable the caching mechanism "
-        "and may slow down the shell",
-        doc_default="True",
-    )
-
     HOSTNAME = Var.with_default(
         default=default_value(lambda env: platform.node()),
         doc="Automatically set to the name of the current host.",
@@ -883,19 +876,13 @@ class GeneralSetting(Xettings):
         doc="Automatically set to a string that fully describes the system type on which xonsh is executing.",
         type_str="str",
     )
-    LANG = Var.with_default(
-        default="C.UTF-8",
-        doc="Fallback locale setting for systems where it matters",
-        type_str="str",
-    )
-    LC_COLLATE = Var.for_locale("LC_COLLATE")
-    LC_CTYPE = Var.for_locale("LC_CTYPE")
-    LC_MONETARY = Var.for_locale("LC_MONETARY")
-    LC_NUMERIC = Var.for_locale("LC_NUMERIC")
-    LC_TIME = Var.for_locale("LC_TIME")
-    if hasattr(locale, "LC_MESSAGES"):
-        LC_MESSAGES = Var.for_locale("LC_MESSAGES")
 
+    PWD = Var.with_default(
+        _get_cwd() or ".",
+        "Current working directory.",
+        "os.getcwd()",
+        is_configurable=False,
+    )
     OLDPWD = Var.with_default(
         ".",
         "Used to represent a previous present working directory.",
@@ -925,18 +912,6 @@ class GeneralSetting(Xettings):
         "filtering valid executables by. Each element must be "
         "uppercase.",
     )
-    RAISE_SUBPROC_ERROR = Var.with_default(
-        False,
-        "Whether or not to raise an error if a subprocess (captured or "
-        "uncaptured) returns a non-zero exit status, which indicates failure. "
-        "This is most useful in xonsh scripts or modules where failures "
-        "should cause an end to execution. This is less useful at a terminal. "
-        "The error that is raised is a ``subprocess.CalledProcessError``.",
-    )
-    LAST_RETURN_CODE = Var.with_default(
-        0,
-        "Integer return code of the last command. Only updated during interactive use, i.e. not during execution of scripts.",
-    )
 
     SHLVL = Var(
         is_valid_shlvl,
@@ -946,10 +921,7 @@ class GeneralSetting(Xettings):
         "Shell nesting level typed as integer, mirrors bash's $SHLVL.",
         is_configurable=False,
     )
-    XONSH_SUBPROC_CAPTURED_PRINT_STDERR = Var.with_default(
-        False,
-        "If ``True`` the stderr from captured subproc will be printed automatically.",
-    )
+
     TERM = Var.no_default(
         "str",
         "TERM is sometimes set by the terminal emulator. This is used (when "
@@ -968,41 +940,7 @@ class GeneralSetting(Xettings):
         "not always happen.",
         is_configurable=False,
     )
-    XONSH_CAPTURE_ALWAYS = Var.with_default(
-        False,
-        "Try to capture output of commands run without explicit capturing.\n"
-        "If True, xonsh will capture the output of commands run directly or in ``![]``"
-        "to the session history.\n"
-        "Setting to True has the following disadvantages:\n"
-        "* Some interactive commands won't work properly (like when ``git`` invokes an interactive editor).\n"
-        "  For more information see discussion at https://github.com/xonsh/xonsh/issues/3672.\n"
-        "* Stopping these commands with ^Z (i.e. ``SIGTSTP``)\n"
-        "  is disabled as it causes deadlocked terminals.\n"
-        "  ``SIGTSTP`` may still be issued and only the physical pressing\n"
-        "  of ``Ctrl+Z`` is ignored.\n\n"
-        "Regardless of this value, commands run in ``$()``, ``!()`` or with an IO redirection (``>`` or ``|``) "
-        "will always be captured.\n"
-        "Setting this to True depends on ``$THREAD_SUBPROCS`` being True.",
-    )
-    THREAD_SUBPROCS = Var(
-        is_bool_or_none,
-        to_bool_or_none,
-        bool_or_none_to_str,
-        not ON_CYGWIN,
-        "Note: The ``$XONSH_CAPTURE_ALWAYS`` variable introduces finer control "
-        "and you should probably use that instead.\n\n"
-        "Whether or not to try to run subrocess mode in a Python thread, "
-        "when trying to capture its output. There are various trade-offs.\n\n"
-        "If True, xonsh is able capture & store the stdin, stdout, and stderr \n"
-        "  of threadable subprocesses.\n"
-        "The disadvantages are listed in ``$XONSH_CAPTURE_ALWAYS``.\n"
-        "The desired effect is often up to the command, user, or use case.\n\n"
-        "None values are for internal use only and are used to turn off "
-        "threading when loading xonshrc files. This is done because Bash "
-        "was automatically placing new xonsh instances in the background "
-        "at startup when threadable subprocs were used. Please see "
-        "https://github.com/xonsh/xonsh/pull/3705 for more information.\n",
-    )
+
     UPDATE_OS_ENVIRON = Var.with_default(
         False,
         "If True ``os_environ`` will always be updated "
@@ -1035,6 +973,17 @@ class GeneralSetting(Xettings):
         "A list of directories where system level data files are stored.",
         type_str="env_path",
     )
+    XONSH_CONFIG_DIR = Var.with_default(
+        xonsh_config_dir,
+        "This is the location where xonsh user-level configuration information is stored.",
+        type_str="str",
+    )
+    XONSH_SYS_CONFIG_DIR = Var.with_default(
+        xonsh_sys_config_dir,
+        "This is the location where xonsh system-level configuration information is stored.",
+        is_configurable=False,
+        type_str="str",
+    )
     XONSHRC = Var.with_default(
         default_xonshrc,
         "A list of the locations of run control files, if they exist.  User "
@@ -1050,57 +999,12 @@ class GeneralSetting(Xettings):
         "are loaded after any files in XONSHRC.",
         type_str="env_path",
     )
-    XONSH_COMPLETER_DIRS = Var.with_default(
-        default_completer_dirs,
-        """\
-A list of paths where Xonsh searches for command completions.
-Any completions defined are lazy loaded when needed.
-The name of the completer file should match that of the completing command.
-The file should contain a function with the signature
-``xonsh_complete(ctx: CommandContext) -> Iterator[RichCompletion|str]``.
-""",
-        type_str="env_path",
-    )
-    XONSH_APPEND_NEWLINE = Var.with_default(
-        xonsh_append_newline,
-        "Append new line when a partial line is preserved in output.",
-        doc_default="``$XONSH_INTERACTIVE``",
-        type_str="bool",
-    )
-    XONSH_CACHE_SCRIPTS = Var.with_default(
-        True,
-        "Controls whether the code for scripts run from xonsh will be cached"
-        " (``True``) or re-compiled each time (``False``).",
-    )
-    XONSH_CACHE_EVERYTHING = Var.with_default(
-        False,
-        "Controls whether all code (including code entered at the interactive"
-        " prompt) will be cached.",
-    )
-    XONSH_CONFIG_DIR = Var.with_default(
-        xonsh_config_dir,
-        "This is the location where xonsh user-level configuration information is stored.",
-        is_configurable=False,
-        type_str="str",
-    )
-    XONSH_SYS_CONFIG_DIR = Var.with_default(
-        xonsh_sys_config_dir,
-        "This is the location where xonsh system-level configuration information is stored.",
-        is_configurable=False,
-        type_str="str",
-    )
     XONSH_COLOR_STYLE = Var.with_default(
         "default",
         "Sets the color style for xonsh colors. This is a style name, not "
         "a color map. Run ``xonfig styles`` to see the available styles.",
         type_str="str",
     )
-    XONSH_DATETIME_FORMAT = Var.with_default(
-        "%Y-%m-%d %H:%M",
-        "The format that is used for ``datetime.strptime()`` in various places, "
-        "i.e the history timestamp option.",
-        type_str="str",
-    )
     XONSH_DEBUG = Var(
         always_false,
         to_debug,
@@ -1118,12 +1022,6 @@ The file should contain a function with the signature
         doc_default="``$XDG_DATA_HOME/xonsh``",
         type_str="str",
     )
-    XONSH_CACHE_DIR = Var.with_default(
-        xonsh_cache_dir,
-        "This is the location where cache files used by xonsh are stored, such as commands-cache...",
-        doc_default="``$XDG_CACHE_HOME/xonsh``",
-        type_str="str",
-    )
     XONSH_ENCODING = Var.with_default(
         DEFAULT_ENCODING,
         "This is the encoding that xonsh should use for subprocess operations.",
@@ -1142,7 +1040,9 @@ The file should contain a function with the signature
     )
     XONSH_INTERACTIVE = Var.with_default(
         True,
-        "``True`` if xonsh is running interactively, and ``False`` otherwise.",
+        "``True`` if xonsh is running interactively, and ``False`` otherwise. "
+        "It's highly recommended to use this variable in your ``xonshrc`` files "
+        "to split the code execution for interactive and non-interactive modes.",
         is_configurable=False,
     )
     XONSH_LOGIN = Var.with_default(
@@ -1150,18 +1050,14 @@ The file should contain a function with the signature
         "``True`` if xonsh is running as a login shell, and ``False`` otherwise.",
         is_configurable=False,
     )
-    XONSH_PROC_FREQUENCY = Var.with_default(
-        1e-4,
-        "The process frequency is the time that "
-        "xonsh process threads sleep for while running command pipelines. "
-        "The value has units of seconds [s].",
-    )
-    XONSH_SHOW_TRACEBACK = Var.with_default(
-        False,
-        "Controls if a traceback is shown if exceptions occur in the shell. "
-        "Set to ``True`` to always show traceback or ``False`` to always hide. "
-        "If undefined then the traceback is hidden but a notice is shown on how "
-        "to enable the full traceback.",
+    XONSH_MODE = Var.with_default(
+        default="interactive",  # In sync with ``main.py``.
+        doc="A string value representing the current xonsh execution mode: "
+        "``interactive``, ``script_from_file``, ``source``, ``single_command``, ``script_from_stdin``."
+        "Note! This variable reflects the mode at start time  (e.g. ``script_from_file``) "
+        "or code execution (e.g. ``source``).  If you need to gate behavior in an RC file that "
+        "you plan to ``source``, use ``$XONSH_INTERACTIVE`` as the flag instead.",
+        type_str="str",
     )
     XONSH_SOURCE = Var.with_default(
         "",
@@ -1186,13 +1082,89 @@ The file should contain a function with the signature
         "    - ptk style name (string) - ``$XONSH_STYLE_OVERRIDES['pygments.keyword'] = '#ff0000'``\n\n"
         "(The rules above are all have the same effect.)",
     )
-    XONSH_TRACE_SUBPROC = Var.with_default(
+    STAR_PATH = Var.no_default("env_path", pattern=re.compile(r"\w*PATH$"))
+    STAR_DIRS = Var.no_default("env_path", pattern=re.compile(r"\w*DIRS$"))
+
+
+class SubprocessSetting(Xettings):
+    """Subprocess Settings"""
+
+    RAISE_SUBPROC_ERROR = Var.with_default(
         False,
-        "Set to ``True`` to show arguments list of every executed subprocess command.",
+        "Whether or not to raise an error if a subprocess (captured or "
+        "uncaptured) returns a non-zero exit status, which indicates failure. "
+        "This is most useful in xonsh scripts or modules where failures "
+        "should cause an end to execution. This is less useful at a terminal. "
+        "The error that is raised is a ``subprocess.CalledProcessError``.",
     )
-    XONSH_TRACE_COMPLETIONS = Var.with_default(
+    LAST_RETURN_CODE = Var.with_default(
+        0,
+        "Integer return code of the last command. Only updated during interactive use, i.e. not during execution of scripts.",
+    )
+    XONSH_SUBPROC_CAPTURED_PRINT_STDERR = Var.with_default(
         False,
-        "Set to ``True`` to show completers invoked and their return values.",
+        "If ``True`` the stderr from captured subproc will be printed automatically.",
+    )
+    XONSH_SUBPROC_OUTPUT_FORMAT = Var.with_default(
+        "stream_lines",
+        "Set output format for subprocess e.g. ``du $(ls)``. "
+        "By default (``stream_lines``) subprocess operator returns text output. "
+        "Set ``list_lines`` to have list of lines.",
+    )
+
+    XONSH_CAPTURE_ALWAYS = Var.with_default(
+        False,
+        "Try to capture output of commands run without explicit capturing.\n"
+        "If True, xonsh will capture the output of commands run directly or in ``![]`` "
+        "to the session history.\n"
+        "Setting to True has the following disadvantages:\n"
+        "* Some interactive commands won't work properly (like when ``git`` invokes an interactive editor).\n"
+        "  For more information see discussion at https://github.com/xonsh/xonsh/issues/3672.\n"
+        "* Stopping these commands with ^Z (i.e. ``SIGTSTP``)\n"
+        "  is disabled as it causes deadlocked terminals.\n"
+        "  ``SIGTSTP`` may still be issued and only the physical pressing\n"
+        "  of ``Ctrl+Z`` is ignored.\n\n"
+        "Regardless of this value, commands run in ``$()``, ``!()`` or with an IO redirection (``>`` or ``|``) "
+        "will always be captured.\n"
+        "Setting this to True depends on ``$THREAD_SUBPROCS`` being True.",
+    )
+    THREAD_SUBPROCS = Var(
+        is_bool_or_none,
+        to_bool_or_none,
+        bool_or_none_to_str,
+        not ON_CYGWIN,
+        "Note: The ``$XONSH_CAPTURE_ALWAYS`` variable introduces finer control "
+        "and you should probably use that instead.\n\n"
+        "Whether or not to try to run subrocess mode in a Python thread, "
+        "when trying to capture its output. There are various trade-offs.\n\n"
+        "If True, xonsh is able capture & store the stdin, stdout, and stderr"
+        " of threadable subprocesses.\n"
+        "The disadvantages are listed in ``$XONSH_CAPTURE_ALWAYS``.\n"
+        "The desired effect is often up to the command, user, or use case.\n\n"
+        "None values are for internal use only and are used to turn off "
+        "threading when loading xonshrc files. This is done because Bash "
+        "was automatically placing new xonsh instances in the background "
+        "at startup when threadable subprocs were used. Please see "
+        "https://github.com/xonsh/xonsh/pull/3705 for more information.\n",
+    )
+    XONSH_APPEND_NEWLINE = Var.with_default(
+        xonsh_append_newline,
+        "Append new line when a partial line is preserved in output.",
+        doc_default="``$XONSH_INTERACTIVE``",
+        type_str="bool",
+    )
+    XONSH_PROC_FREQUENCY = Var.with_default(
+        1e-4,
+        "The process frequency is the time that "
+        "xonsh process threads sleep for while running command pipelines. "
+        "The value has units of seconds [s].",
+    )
+    XONSH_TRACE_SUBPROC = Var(
+        default=False,
+        validate=is_bool_or_int,
+        convert=to_bool_or_int,
+        doc="Set to ``True`` or ``1`` to show arguments list of every executed subprocess command. "
+        "Use ``2`` to have a specification. Use ``3`` to have full specification.",
     )
     XONSH_TRACE_SUBPROC_FUNC = Var.with_default(
         None,
@@ -1200,13 +1172,25 @@ The file should contain a function with the signature
             "A callback function used to format the trace output shown when $XONSH_TRACE_SUBPROC=True."
         ),
         doc_default="""\
-By default it just prints ``cmds`` like below.
+    By default it just prints ``cmds`` like below.
 
-.. code-block:: python
+    .. code-block:: python
 
-    def tracer(cmds: list, captured: Union[bool, str]):
-        print(f"TRACE SUBPROC: {cmds}, captured={captured}", file=sys.stderr)
-""",
+        def tracer(cmds: list, captured: Union[bool, str]):
+            print(f"TRACE SUBPROC: {cmds}, captured={captured}", file=sys.stderr)
+    """,
+    )
+
+
+class ErrorHandlingSetting(Xettings):
+    """Error Handling Settings"""
+
+    XONSH_SHOW_TRACEBACK = Var.with_default(
+        False,
+        "Controls if a traceback is shown if exceptions occur in the shell. "
+        "Set to ``True`` to always show traceback or ``False`` to always hide. "
+        "If undefined then the traceback is hidden but a notice is shown on how "
+        "to enable the full traceback.",
     )
     XONSH_TRACEBACK_LOGFILE = Var(
         is_logfile_opt,
@@ -1218,19 +1202,79 @@ By default it just prints ``cmds`` like below.
         "or None / the empty string if traceback logging is not desired. "
         "Logging to a file is not enabled by default.",
     )
-    XONTRIBS_AUTOLOAD_DISABLED = Var(
-        default=False,
-        doc="""\
-Controls auto-loading behaviour of xontrib packages at the startup.
-* Set this to ``True`` to disable autoloading completely.
-* Setting this to a list of xontrib names will block loading those specifically.
-""",
-        doc_default="""\
-Xontribs with ``xonsh.xontrib`` entrypoint will be loaded automatically by default.
-""",
+
+
+class JobsSetting(Xettings):
+    """Jobs Settings"""
+
+    AUTO_CONTINUE = Var.with_default(
+        False,
+        "If ``True``, automatically resume stopped jobs when they are disowned. "
+        "When stopped jobs are disowned and this option is ``False``, a warning "
+        "will print information about how to continue the stopped process.",
+    )
+
+
+class LangSetting(Xettings):
+    """Language and locale settings."""
+
+    LANG = Var.with_default(
+        default="C.UTF-8",
+        doc="Fallback locale setting for systems where it matters",
+        type_str="str",
+    )
+    LC_COLLATE = Var.for_locale("LC_COLLATE")
+    LC_CTYPE = Var.for_locale("LC_CTYPE")
+    LC_MONETARY = Var.for_locale("LC_MONETARY")
+    LC_NUMERIC = Var.for_locale("LC_NUMERIC")
+    LC_TIME = Var.for_locale("LC_TIME")
+    if hasattr(locale, "LC_MESSAGES"):
+        LC_MESSAGES = Var.for_locale("LC_MESSAGES")
+
+    XONSH_DATETIME_FORMAT = Var.with_default(
+        "%Y-%m-%d %H:%M",
+        "The format that is used for ``datetime.strptime()`` in various places, "
+        "i.e the history timestamp option.",
+        type_str="str",
+    )
+
+
+class CacheSetting(Xettings):
+    """Cache Settings"""
+
+    XONSH_CACHE_SCRIPTS = Var.with_default(
+        True,
+        "Controls whether the code for scripts run from xonsh will be cached"
+        " (``True``) or re-compiled each time (``False``).",
+    )
+
+    XONSH_CACHE_EVERYTHING = Var.with_default(
+        False,
+        "Controls whether all code (including code entered at the interactive"
+        " prompt) will be cached.",
+    )
+
+    XONSH_CACHE_DIR = Var.with_default(
+        xonsh_cache_dir,
+        "This is the location where cache files used by xonsh are stored, such as commands-cache...",
+        doc_default="``$XDG_CACHE_HOME/xonsh``",
+        type_str="str",
+    )
+
+    ENABLE_COMMANDS_CACHE = Var(
+        default=True,
+        doc="command names in a directory are cached when enabled. "
+        "On some platforms it may not be accurate enough"
+        "(e.g. Windows, Linux save mtime in seconds). "
+        "Setting it to False would disable the caching mechanism "
+        "and may slow down the shell",
+        doc_default="True",
+    )
+
+    COMMANDS_CACHE_SAVE_INTERMEDIATE = Var.with_default(
+        False,
+        "If enabled, the CommandsCache is saved between runs and can reduce the startup time.",
     )
-    STAR_PATH = Var.no_default("env_path", pattern=re.compile(r"\w*PATH$"))
-    STAR_DIRS = Var.no_default("env_path", pattern=re.compile(r"\w*DIRS$"))
 
 
 class ChangeDirSetting(Xettings):
@@ -1311,6 +1355,23 @@ class InterpreterSetting(Xettings):
     )
 
 
+class XontribSetting(Xettings):
+    """Xontrib Settings"""
+
+    XONTRIBS_AUTOLOAD_DISABLED = Var.with_default(
+        default=False,
+        type_str="bool",
+        doc="""\
+    Controls auto-loading behaviour of xontrib packages at the startup.
+    * Set this to ``True`` to disable autoloading completely.
+    * Setting this to a list of xontrib names will block loading those specifically.
+    """,
+        doc_default="""\
+    Xontribs with ``xonsh.xontrib`` entrypoint will be loaded automatically by default.
+    """,
+    )
+
+
 class PromptSetting(Xettings):
     """Interactive Prompt"""
 
@@ -1383,7 +1444,7 @@ class PromptSetting(Xettings):
         is_string_or_callable,
         ensure_string,
         ensure_string,
-        ".",
+        " ",
         "Prompt text for 2nd+ lines of input, may be str or function which "
         "returns a str.",
     )
@@ -1485,7 +1546,7 @@ class PromptSetting(Xettings):
         "`prompt_toolkit <https://github.com/jonathanslenders/python-prompt-toolkit>`_"
         " library installed. To specify which shell should be used, do so in "
         "the run control file. "
-        "It also accepts a class type that inherits from ``xonsh.base_shell.BaseShell``",
+        "It also accepts a class type that inherits from ``xonsh.shells.base_shell.BaseShell``",
         doc_default="``best``",
     )
     SUGGEST_COMMANDS = Var.with_default(
@@ -1567,6 +1628,10 @@ class PromptSetting(Xettings):
         "For example, to have stderr appear on a red background, the "
         'prefix & postfix pair would be "{BACKGROUND_RED}" & "{RESET}".',
     )
+    XONSH_SUPPRESS_WELCOME = Var.with_default(
+        False,
+        "Suppresses the welcome message.",
+    )
 
 
 class PromptHistorySetting(Xettings):
@@ -1639,6 +1704,11 @@ class PromptHistorySetting(Xettings):
         "Example: ``$XONSH_HISTORY_IGNORE_REGEX = '(^echo|^.*\\#\\#\\#\\s*|.*\\#\\#\\#\\s*$)'``"
         " - skip commands that start from ``echo`` or ``###``, or end from ``###``.",
     )
+    XONSH_HISTORY_SIGINT_FLUSH = Var.with_default(
+        True,
+        "Save history after getting SIGINT (Ctrl+C).",
+        doc_default="True",
+    )
 
 
 class PTKSetting(PromptSetting):  # sub-classing -> sub-group
@@ -1648,8 +1718,8 @@ class PTKSetting(PromptSetting):  # sub-classing -> sub-group
 
     AUTO_SUGGEST = Var.with_default(
         True,
-        "Enable automatic command suggestions based on history, like in the fish "
-        "shell.\n\nPressing the right arrow key inserts the currently "
+        "Enable automatic command suggestions based on history."
+        "\n\nPressing the right arrow key inserts the currently "
         "displayed suggestion. ",
     )
     AUTO_SUGGEST_IN_COMPLETIONS = Var.with_default(
@@ -1674,6 +1744,19 @@ class PTKSetting(PromptSetting):  # sub-classing -> sub-group
         "``DEPTH_1_BIT``, ``DEPTH_4_BIT``, ``DEPTH_8_BIT``, ``DEPTH_24_BIT`` "
         "colors. Default is an empty string which means that prompt toolkit decide.",
     )
+    XONSH_PROMPT_CURSOR_SHAPE = Var(
+        always_false,
+        to_ptk_cursor_shape,
+        to_ptk_cursor_shape_display_value,
+        to_ptk_cursor_shape("modal-vi-mode-only"),
+        "The cursor shape. Possible values for prompt toolkit are: "
+        "``block``, ``beam``, ``underline``, "
+        "``blinking-block``, ``blinking-beam``, ``blinking-underline``, "
+        "``modal``, ``modal-vi-mode-only``, ``never-change``. "
+        "Default value is ``modal-vi-mode-only`` which means "
+        "``modal`` if in vi mode and ``never-change`` if not in vi mode.",
+        doc_default="modal-vi-mode-only",
+    )
     PTK_STYLE_OVERRIDES = Var(
         is_tok_color_dict,
         to_tok_color_dict,
@@ -1786,7 +1869,7 @@ This is to reduce the noise in generated completions.""",
     )
     CASE_SENSITIVE_COMPLETIONS = Var.with_default(
         ON_LINUX,
-        "Sets whether completions should be case sensitive or case " "insensitive.",
+        "Sets whether completions should be case sensitive or case insensitive.",
         doc_default="True on Linux, False otherwise.",
     )
     COMPLETIONS_BRACKETS = Var.with_default(
@@ -1821,6 +1904,21 @@ This is to reduce the noise in generated completions.""",
         "``xonsh.platform.bash_command``.",
         doc_default="None",
     )
+    XONSH_COMPLETER_DIRS = Var.with_default(
+        default_completer_dirs,
+        """\
+A list of paths where Xonsh searches for command completions.
+Any completions defined are lazy loaded when needed.
+The name of the completer file should match that of the completing command.
+The file should contain a function with the signature
+``xonsh_complete(ctx: CommandContext) -> Iterator[RichCompletion|str]``.
+""",
+        type_str="env_path",
+    )
+    XONSH_TRACE_COMPLETIONS = Var.with_default(
+        False,
+        "Set to ``True`` to show completers invoked and their return values.",
+    )
 
 
 class PTKCompletionSetting(AutoCompletionSetting):
@@ -1964,7 +2062,7 @@ class Env(cabc.MutableMapping):
         if "PATH" not in self._d:
             # this is here so the PATH is accessible to subprocs and so that
             # it can be modified in-place in the xonshrc file
-            self._d["PATH"] = list(PATH_DEFAULT)
+            self._d["PATH"] = EnvPath(PATH_DEFAULT)
         self._detyped = None
 
     def get_detyped(self, key: str):
@@ -1972,7 +2070,11 @@ class Env(cabc.MutableMapping):
         return detyped.get(key)
 
     def detype(self):
-        """return a dict that can be used as ``os.environ``"""
+        """
+        Returns a dict of detyped variables.
+        Note! If env variable wasn't explicitly set (e.g. the value has default value in ``Xettings``)
+        it will be not in this list.
+        """
         if self._detyped is not None:
             return self._detyped
         ctx = {}
@@ -1991,6 +2093,24 @@ class Env(cabc.MutableMapping):
         self._detyped = ctx
         return ctx
 
+    def detype_all(self):  # __getitem__
+        """Returns a dict of all available detyped env variables."""
+        if self._detyped is not None:
+            return self._detyped
+        ctx = {}
+        for key in self.rawkeys():
+            if not isinstance(key, str):
+                key = str(key)
+            val = self.__getitem__(key)
+            detyper = self.get_detyper(key)
+            if detyper is not None:
+                val = detyper(val)
+            if not isinstance(val, str):
+                continue
+            ctx[key] = val
+        self._detyped = ctx
+        return ctx
+
     def replace_env(self):
         """Replaces the contents of os_environ with a detyped version
         of the xonsh environment.
@@ -2182,7 +2302,7 @@ class Env(cabc.MutableMapping):
             e = "Unknown environment variable: ${}"
             raise KeyError(e.format(key))
         if isinstance(
-            val, (cabc.MutableSet, cabc.MutableSequence, cabc.MutableMapping)
+            val, cabc.MutableSet | cabc.MutableSequence | cabc.MutableMapping
         ):
             self._detyped = None
         return val
@@ -2455,22 +2575,13 @@ class InternalEnvironDict(ChainMap):
         local.update(new_local)
 
 
-def _yield_executables(directory, name):
-    if ON_WINDOWS:
-        base_name, ext = os.path.splitext(name.lower())
-        for fname in executables_in(directory):
-            fbase, fext = os.path.splitext(fname.lower())
-            if base_name == fbase and (len(ext) == 0 or ext == fext):
-                yield os.path.join(directory, fname)
-    else:
-        for x in executables_in(directory):
-            if x == name:
-                yield os.path.join(directory, name)
-                return
-
-
 def locate_binary(name):
-    """Locates an executable on the file system."""
+    """Locates an executable on the file system.
+
+    NOT RECOMMENDED because ``commands_cache.locate_binary`` contains ``update_cache``
+    with scanning all files in ``$PATH``. First of all take a look into ``xonsh.specs.executables``
+    for more fast implementation the locate operation.
+    """
     return XSH.commands_cache.locate_binary(name)
 
 
