@@ -19,9 +19,9 @@ from xonsh.built_ins import XSH, XonshSession
 from xonsh.completer import Completer
 from xonsh.events import events
 from xonsh.execer import Execer
-from xonsh.jobs import get_tasks
 from xonsh.main import setup
 from xonsh.parsers.completion_context import CompletionContextParser
+from xonsh.procs.jobs import get_tasks
 
 from .tools import DummyHistory, DummyShell, copy_env, sp
 
@@ -34,14 +34,7 @@ def pytest_collection_modifyitems(items):
     """Move xsh test first to work around a bug in normal
     pytest cleanup. The order of tests are otherwise preserved.
     """
-    xsh_items = []
-    other_items = []
-    for item in items:
-        if isinstance(item, XshFunction):
-            xsh_items.append(item)
-        else:
-            other_items.append(item)
-    items[:] = xsh_items + other_items
+    items.sort(key=lambda item: -isinstance(item, XshFunction))
 
 
 def _limited_traceback(excinfo):
@@ -139,8 +132,8 @@ def mock_executables_in(xession, tmp_path, monkeypatch):
 
 @pytest.fixture
 def patch_locate_binary(monkeypatch):
-    def locate_binary(self, name):
-        return os.path.join(os.path.dirname(__file__), "bin", name)
+    def locate_binary(self, name, *args):
+        return str(Path(__file__).parent.parent.parent / "tests" / "bin" / name)
 
     def factory(cc: commands_cache.CommandsCache):
         monkeypatch.setattr(cc, "locate_binary", types.MethodType(locate_binary, cc))
@@ -370,7 +363,7 @@ def ptk_shell(xonsh_execer):
     from prompt_toolkit.input import create_pipe_input
     from prompt_toolkit.output import DummyOutput
 
-    from xonsh.ptk_shell.shell import PromptToolkitShell
+    from xonsh.shells.ptk_shell import PromptToolkitShell
 
     out = DummyOutput()
     with create_pipe_input() as inp:
@@ -382,7 +375,7 @@ def ptk_shell(xonsh_execer):
 
 @pytest.fixture
 def readline_shell(xonsh_execer, tmpdir, mocker):
-    from xonsh.readline_shell import ReadlineShell
+    from xonsh.shells.readline_shell import ReadlineShell
 
     inp_path = tmpdir / "in"
     inp = inp_path.open("w+")
